@@ -120,7 +120,6 @@ static void         update_packing                          (MprisMenuItem  *ite
 /* Public API */
 GtkWidget*
 mpris_menu_item_new_with_player (const gchar *player,
-                                 const gchar *title,
                                  const gchar *icon_name,
                                  const gchar *filename)
 {
@@ -134,10 +133,7 @@ mpris_menu_item_new_with_player (const gchar *player,
   priv = mpris_menu_item_get_instance_private (menu_item);
 
   priv->player = g_strdup(player);
-  if (title != NULL)
-    priv->title = g_strdup(title);
-  else
-    priv->title = g_strdup(player);
+  priv->title = g_strdup(player);
   priv->filename = g_strdup(filename);
 
   update_packing (menu_item);
@@ -175,16 +171,15 @@ GtkWidget*
 mpris_menu_item_new_from_player_name (const gchar *player)
 {
   GtkWidget *widget = NULL;
-  gchar     *name;
   gchar     *icon_name;
   gchar     *full_path;
 
-  if (pulseaudio_mpris_get_player_summary (player, &name, &icon_name, &full_path)) {
-    widget = mpris_menu_item_new_with_player (player, name, icon_name, full_path);
-    g_free (name);
-    g_free (icon_name);
-    g_free (full_path);
-  }
+  if (pulseaudio_mpris_get_player_summary (player, &icon_name, &full_path))
+    {
+      widget = mpris_menu_item_new_with_player (player, icon_name, full_path);
+      g_free (icon_name);
+      g_free (full_path);
+    }
 
   return widget;
 }
@@ -274,7 +269,12 @@ mpris_menu_item_set_can_play (MprisMenuItem *item,
   priv->can_play = enabled;
 
   if (priv->is_running)
-    gtk_widget_set_sensitive (priv->play_pause, priv->can_play);
+    {
+      if (!priv->is_playing)
+        {
+          gtk_widget_set_sensitive (priv->play_pause, priv->can_play);
+        }
+    }
   else
     gtk_widget_set_sensitive (priv->play_pause, FALSE);
 }
@@ -482,22 +482,6 @@ mpris_menu_item_class_init (MprisMenuItemClass *item_class)
 static void
 mpris_menu_item_init (MprisMenuItem *item)
 {
-  MprisMenuItemPrivate *priv;
-
-  priv = mpris_menu_item_get_instance_private (item);
-
-  priv->title_label = NULL;
-  priv->artist_label = NULL;
-  priv->button_box = NULL;
-  priv->vbox = NULL;
-  priv->hbox = NULL;
-  priv->go_previous = NULL;
-  priv->play_pause = NULL;
-  priv->go_next = NULL;
-
-  priv->player = NULL;
-  priv->title = NULL;
-  priv->filename = NULL;
 }
 
 
@@ -511,12 +495,9 @@ mpris_menu_item_finalize (GObject *object)
   item = MPRIS_MENU_ITEM (object);
   priv = mpris_menu_item_get_instance_private (item);
 
-  if (priv->player)
-    g_free (priv->player);
-  if (priv->title)
-    g_free (priv->title);
-  if (priv->filename)
-    g_free (priv->filename);
+  g_free (priv->player);
+  g_free (priv->title);
+  g_free (priv->filename);
 
   g_object_unref (priv->title_label);
   g_object_unref (priv->artist_label);
@@ -569,7 +550,7 @@ mpris_menu_item_launch (MprisMenuItem *item)
 
   priv = mpris_menu_item_get_instance_private (item);
 
-  if (priv->is_running)
+  if (priv->is_running || !priv->filename)
     return;
 
   app_info = (GAppInfo*)g_desktop_app_info_new_from_filename (priv->filename);
