@@ -109,6 +109,7 @@ struct _PulseaudioButton
 
   PulseaudioMenu       *menu;
 
+  gulong                connection_changed_id;
   gulong                volume_changed_id;
   gulong                recording_volume_changed_id;
   gulong                recording_changed_id;
@@ -201,8 +202,6 @@ pulseaudio_button_init (PulseaudioButton *button)
                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
   g_object_unref (css_provider);
   gtk_style_context_add_class (context, "recording-indicator");
-
-  g_object_set (G_OBJECT (button), "has-tooltip", TRUE, NULL);
 }
 
 
@@ -323,7 +322,9 @@ pulseaudio_query_tooltip (GtkWidget  *widget,
                           GtkTooltip *tooltip)
 {
   PulseaudioButton *button = PULSEAUDIO_BUTTON (widget);
+  const gchar      *format;
   gchar            *tip_text;
+  const gchar      *dev_name;
   gboolean          muted;
   gdouble           volume;
 
@@ -338,22 +339,25 @@ pulseaudio_query_tooltip (GtkWidget  *widget,
     {
       if (pulseaudio_button_mic_icon_under_pointer (button, x))
         {
+          dev_name = pulseaudio_volume_get_input_by_name (button->volume, pulseaudio_volume_get_default_input (button->volume), NULL);
           muted = pulseaudio_volume_get_muted_mic (button->volume);
           volume = pulseaudio_volume_get_volume_mic (button->volume);
         }
       else
         {
+          dev_name = pulseaudio_volume_get_output_by_name (button->volume, pulseaudio_volume_get_default_output (button->volume), NULL);
           muted = pulseaudio_volume_get_muted (button->volume);
           volume = pulseaudio_volume_get_volume (button->volume);
         }
 
       if (muted)
-        tip_text = g_strdup_printf (_("Volume %d%% (muted)"), (gint) round (volume * 100));
+        format = _("<b>Volume %d%% (muted)</b>\n<small>%s</small>");
       else
-        tip_text = g_strdup_printf (_("Volume %d%%"), (gint) round (volume * 100));
+        format = _("<b>Volume %d%%</b>\n<small>%s</small>");
+      tip_text = g_strdup_printf (format, (gint) round (volume * 100), dev_name);
     }
 
-  gtk_tooltip_set_text (tooltip, tip_text);
+  gtk_tooltip_set_markup (tooltip, tip_text);
   g_free (tip_text);
 
   return TRUE;
@@ -395,23 +399,6 @@ pulseaudio_button_update_icons (PulseaudioButton *button)
 
 
 
-static gboolean
-pulseaudio_button_sink_connection_timeout  (gpointer userdata)
-{
-  PulseaudioButton *button = PULSEAUDIO_BUTTON (userdata);
-  gboolean sink_connected = pulseaudio_volume_get_sink_connected (button->volume);
-
-  if (sink_connected)
-  {
-    pulseaudio_button_update (button, TRUE);
-    return FALSE;
-  }
-
-  return TRUE;
-}
-
-
-
 static void
 pulseaudio_button_update (PulseaudioButton *button,
                           gboolean          force_update)
@@ -419,7 +406,6 @@ pulseaudio_button_update (PulseaudioButton *button,
   gdouble      volume;
   gdouble      recording_volume;
   gboolean     connected;
-  gboolean     sink_connected;
   gboolean     muted;
   gboolean     recording_muted;
   gboolean     recording;
@@ -432,7 +418,6 @@ pulseaudio_button_update (PulseaudioButton *button,
   volume = pulseaudio_volume_get_volume (button->volume);
   muted = pulseaudio_volume_get_muted (button->volume);
   connected = pulseaudio_volume_get_connected (button->volume);
-  sink_connected = pulseaudio_volume_get_sink_connected (button->volume);
   recording = pulseaudio_volume_get_recording (button->volume);
   recording_volume = pulseaudio_volume_get_volume_mic (button->volume);
   recording_muted = pulseaudio_volume_get_muted_mic (button->volume);
@@ -482,9 +467,6 @@ pulseaudio_button_update (PulseaudioButton *button,
 
   if (gtk_widget_get_visible (button->recording_indicator) != recording)
     gtk_widget_set_visible (button->recording_indicator, recording);
-
-  if (!sink_connected)
-    g_timeout_add (250, pulseaudio_button_sink_connection_timeout, button);
 }
 
 
@@ -535,6 +517,18 @@ pulseaudio_button_volume_changed (PulseaudioButton  *button,
 {
   g_return_if_fail (IS_PULSEAUDIO_BUTTON (button));
 
+  if (pulseaudio_volume_get_connected (button->volume))
+    pulseaudio_button_update (button, FALSE);
+}
+
+
+
+static void
+pulseaudio_button_update2 (PulseaudioButton  *button,
+                           PulseaudioVolume  *volume)
+{
+  g_return_if_fail (IS_PULSEAUDIO_BUTTON (button));
+
   pulseaudio_button_update (button, FALSE);
 }
 
@@ -561,6 +555,9 @@ pulseaudio_button_new (PulseaudioPlugin *plugin,
   button->volume = volume;
   button->config = config;
   button->mpris = mpris;
+  button->connection_changed_id =
+    g_signal_connect_swapped (G_OBJECT (button->volume), "connection-changed",
+                              G_CALLBACK (pulseaudio_button_update2), button);
   button->volume_changed_id =
     g_signal_connect_swapped (G_OBJECT (button->volume), "volume-changed",
                               G_CALLBACK (pulseaudio_button_volume_changed), button);
