@@ -108,15 +108,23 @@ pulseaudio_dialog_mixer_command_changed (PulseaudioDialog *dialog)
 {
   GObject *object;
   gchar   *path;
+  gchar  **argvp = NULL;
+  gboolean sensitive = FALSE;
 
   g_return_if_fail (GTK_IS_BUILDER (dialog));
   g_return_if_fail (IS_PULSEAUDIO_CONFIG (dialog->config));
 
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "button-run-mixer");
   g_return_if_fail (GTK_IS_BUTTON (object));
-  path = g_find_program_in_path (pulseaudio_config_get_mixer_command (dialog->config));
-  gtk_widget_set_sensitive (GTK_WIDGET (object), path != NULL);
-  g_free (path);
+  if (g_shell_parse_argv (pulseaudio_config_get_mixer_command (dialog->config), NULL, &argvp, NULL))
+    {
+      path = g_find_program_in_path (argvp[0]);
+      if (path)
+        sensitive = TRUE;
+      g_free (path);
+      g_strfreev (argvp);
+    }
+  gtk_widget_set_sensitive (GTK_WIDGET (object), sensitive);
 }
 
 
@@ -152,14 +160,14 @@ pulseaudio_dialog_run_mixer (PulseaudioDialog *dialog,
 
 
 static void
-pulseaudio_dialog_player_toggled_cb (GtkCellRendererToggle *toggle, gchar *path, gpointer user_data)
+pulseaudio_dialog_persistent_toggled_cb (GtkCellRendererToggle *toggle, gchar *path, gpointer user_data)
 {
   PulseaudioDialog *dialog = PULSEAUDIO_DIALOG (user_data);
   GtkTreeModel     *model;
   GtkTreePath      *treepath;
   GtkTreeIter       iter;
-  GValue            hidden_val = {0,}, player_val = {0,};
-  gboolean          hidden;
+  GValue            persistent_val = G_VALUE_INIT, player_val = G_VALUE_INIT;
+  gboolean          persistent;
   const gchar      *player;
 
   model = GTK_TREE_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(dialog->treeview)));
@@ -167,16 +175,46 @@ pulseaudio_dialog_player_toggled_cb (GtkCellRendererToggle *toggle, gchar *path,
   gtk_tree_model_get_iter (model, &iter, treepath);
 
   gtk_tree_model_get_value (model, &iter, 1, &player_val);
-  gtk_tree_model_get_value (model, &iter, 3, &hidden_val);
-  hidden = !g_value_get_boolean(&hidden_val);
+  gtk_tree_model_get_value (model, &iter, 2, &persistent_val);
+  persistent = !g_value_get_boolean(&persistent_val);
   player = g_value_get_string(&player_val);
 
-  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 3, hidden, -1);
+  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 2, persistent, -1);
 
-  if (hidden)
-    pulseaudio_config_player_blacklist_add (dialog->config, player);
+  if (persistent)
+    pulseaudio_config_player_persistent_add (dialog->config, player);
   else
-    pulseaudio_config_player_blacklist_remove (dialog->config, player);
+    pulseaudio_config_player_persistent_remove (dialog->config, player);
+}
+
+
+
+static void
+pulseaudio_dialog_ignored_toggled_cb (GtkCellRendererToggle *toggle, gchar *path, gpointer user_data)
+{
+  PulseaudioDialog *dialog = PULSEAUDIO_DIALOG (user_data);
+  GtkTreeModel     *model;
+  GtkTreePath      *treepath;
+  GtkTreeIter       iter;
+  GValue            ignored_val = G_VALUE_INIT, player_val = G_VALUE_INIT;
+  gboolean          ignored;
+  const gchar      *player;
+
+  model = GTK_TREE_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(dialog->treeview)));
+  treepath = gtk_tree_path_new_from_string (path);
+  gtk_tree_model_get_iter (model, &iter, treepath);
+
+  gtk_tree_model_get_value (model, &iter, 1, &player_val);
+  gtk_tree_model_get_value (model, &iter, 3, &ignored_val);
+  ignored = !g_value_get_boolean(&ignored_val);
+  player = g_value_get_string(&player_val);
+
+  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 3, ignored, -1);
+
+  if (ignored)
+    pulseaudio_config_player_ignored_add (dialog->config, player);
+  else
+    pulseaudio_config_player_ignored_remove (dialog->config, player);
 }
 
 
@@ -263,6 +301,12 @@ pulseaudio_dialog_build (PulseaudioDialog *dialog)
                               G_OBJECT (object), "value",
                               G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
+      object = gtk_builder_get_object (builder, "spinbutton-max-volume");
+      g_return_if_fail (GTK_IS_ENTRY (object));
+      g_object_bind_property (G_OBJECT (dialog->config), "volume-max",
+                              G_OBJECT (object), "value",
+                              G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
+
       object = gtk_builder_get_object (builder, "entry-mixer-command");
       g_return_if_fail (GTK_IS_ENTRY (object));
       g_object_bind_property (G_OBJECT (dialog->config), "mixer-command",
@@ -308,17 +352,16 @@ pulseaudio_dialog_build (PulseaudioDialog *dialog)
       /* Populate the liststore */
       dialog->treeview = GTK_WIDGET(gtk_builder_get_object(builder, "player_tree_view"));
       liststore = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(dialog->treeview)));
-      players = pulseaudio_config_get_mpris_players (dialog->config);
+      players = pulseaudio_config_get_known_players (dialog->config);
       if (players != NULL)
         {
-          for (i = 0; i < g_strv_length (players); i++)
+          const guint num_players = g_strv_length (players);
+          for (i = 0; i < num_players; i++)
             {
-              gchar *name = NULL;
               gchar *icon_name = NULL;
-              gchar *full_path = NULL;
               GIcon *icon = NULL;
 
-              if (pulseaudio_mpris_get_player_summary (players[i], &name, &icon_name, &full_path))
+              if (pulseaudio_mpris_get_player_summary (players[i], &icon_name, NULL))
                 {
                   if (g_file_test (icon_name, G_FILE_TEST_EXISTS) && !g_file_test (icon_name, G_FILE_TEST_IS_DIR))
                     {
@@ -339,22 +382,24 @@ pulseaudio_dialog_build (PulseaudioDialog *dialog)
                   gtk_list_store_set (liststore, &iter,
                                       0, icon,
                                       1, players[i],
-                                      2, name,
-                                      3, pulseaudio_config_player_blacklist_lookup (dialog->config, players[i]),
+                                      2, pulseaudio_config_player_persistent_lookup (dialog->config, players[i]),
+                                      3, pulseaudio_config_player_ignored_lookup (dialog->config, players[i]),
                                       -1);
 
-                  g_free (name);
                   g_free (icon_name);
-                  g_free (full_path);
                   if (icon != NULL)
                     g_object_unref (icon);
                 }
             }
-        }
-      g_strfreev (players);
 
-      object = gtk_builder_get_object(builder, "col_hidden_renderer");
-      g_signal_connect (object, "toggled", (GCallback) pulseaudio_dialog_player_toggled_cb, dialog);
+          g_strfreev (players);
+        }
+
+      object = gtk_builder_get_object(builder, "col_persistent_renderer");
+      g_signal_connect (object, "toggled", (GCallback) pulseaudio_dialog_persistent_toggled_cb, dialog);
+
+      object = gtk_builder_get_object(builder, "col_ignored_renderer");
+      g_signal_connect (object, "toggled", (GCallback) pulseaudio_dialog_ignored_toggled_cb, dialog);
 
       object = gtk_builder_get_object(builder, "clear_players");
       g_signal_connect (object, "clicked", (GCallback) pulseaudio_dialog_clear_players_cb, dialog);
