@@ -54,6 +54,7 @@ struct _PulseaudioMenu
   GtkWidget            *output_scale;
   GtkWidget            *input_scale;
 
+  gulong                connection_changed_id;
   gulong                volume_changed_id;
   gulong                volume_mic_changed_id;
 };
@@ -87,14 +88,6 @@ pulseaudio_menu_class_init (PulseaudioMenuClass *klass)
 static void
 pulseaudio_menu_init (PulseaudioMenu *menu)
 {
-  menu->volume                         = NULL;
-  menu->config                         = NULL;
-  menu->mpris                          = NULL;
-  menu->button                         = NULL;
-  menu->output_scale                   = NULL;
-  menu->input_scale                    = NULL;
-  menu->volume_changed_id              = 0;
-  menu->volume_mic_changed_id          = 0;
 }
 
 
@@ -106,21 +99,15 @@ pulseaudio_menu_finalize (GObject *object)
 
   menu = PULSEAUDIO_MENU (object);
 
+  if (menu->connection_changed_id != 0)
+    g_signal_handler_disconnect (G_OBJECT (menu->volume), menu->connection_changed_id);
+
   if (menu->volume_changed_id != 0)
     g_signal_handler_disconnect (G_OBJECT (menu->volume), menu->volume_changed_id);
 
   if (menu->volume_mic_changed_id != 0)
     g_signal_handler_disconnect (G_OBJECT (menu->volume), menu->volume_mic_changed_id);
 
-  menu->volume                         = NULL;
-  menu->config                         = NULL;
-  menu->mpris                          = NULL;
-  menu->button                         = NULL;
-  menu->output_scale                   = NULL;
-  menu->input_scale                    = NULL;
-  menu->volume_changed_id              = 0;
-  menu->volume_mic_changed_id          = 0;
-
   G_OBJECT_CLASS (pulseaudio_menu_parent_class)->finalize (object);
 }
 
@@ -293,6 +280,29 @@ pulseaudio_menu_activate_playlist (PulseaudioMenu *menu,
 
 
 
+static void
+pulseaudio_menu_connection_changed (PulseaudioMenu   *menu,
+                                    PulseaudioVolume *volume)
+{
+  g_return_if_fail(IS_PULSEAUDIO_MENU(menu));
+
+  if (menu->volume_changed_id != 0)
+    {
+      g_signal_handler_disconnect (G_OBJECT (menu->volume), menu->volume_changed_id);
+      menu->volume_changed_id = 0;
+    }
+
+  if (menu->volume_mic_changed_id != 0)
+    {
+      g_signal_handler_disconnect (G_OBJECT (menu->volume), menu->volume_mic_changed_id);
+      menu->volume_mic_changed_id = 0;
+    }
+
+  gtk_widget_set_visible (GTK_WIDGET (menu), FALSE);
+}
+
+
+
 static void
 pulseaudio_menu_volume_changed (PulseaudioMenu   *menu,
                                 gboolean          should_notify,
@@ -357,7 +367,6 @@ mpris_update_cb (PulseaudioMpris *mpris,
 
   gchar          *title;
   gchar          *artist;
-  gboolean        is_running;
   gboolean        is_playing;
   gboolean        is_stopped;
   gboolean        can_play;
@@ -365,7 +374,6 @@ mpris_update_cb (PulseaudioMpris *mpris,
   gboolean        can_go_previous;
   gboolean        can_go_next;
   gboolean        can_raise;
-  GList          *playlists;
 
   g_return_if_fail (IS_PULSEAUDIO_MPRIS (mpris));
   g_return_if_fail (IS_MPRIS_MENU_ITEM (menu_item));
@@ -379,7 +387,6 @@ mpris_update_cb (PulseaudioMpris *mpris,
                                                 player,
                                                 &title,
                                                 &artist,
-                                                &is_running,
                                                 &is_playing,
                                                 &is_stopped,
                                                 &can_play,
@@ -387,9 +394,10 @@ mpris_update_cb (PulseaudioMpris *mpris,
                                                 &can_go_previous,
                                                 &can_go_next,
                                                 &can_raise,
-                                                &playlists))
+                                                NULL))
         {
-          mpris_menu_item_set_is_running (menu_item, is_running);
+          mpris_menu_item_set_is_running (menu_item, TRUE);
+
           mpris_menu_item_set_title (menu_item, title);
           mpris_menu_item_set_artist (menu_item, artist);
 
@@ -401,14 +409,14 @@ mpris_update_cb (PulseaudioMpris *mpris,
 
           mpris_menu_item_set_is_playing (menu_item, is_playing);
           mpris_menu_item_set_is_stopped (menu_item, is_stopped);
-        }
 
-      if (title != NULL)
-        g_free (title);
-      if (artist != NULL)
-        g_free (artist);
-      if (playlists != NULL)
-        g_list_free (playlists);
+          g_free (title);
+          g_free (artist);
+        }
+      else
+        {
+          mpris_menu_item_set_is_running (menu_item, FALSE);
+        }
     }
 }
 
@@ -444,8 +452,11 @@ pulseaudio_menu_new (PulseaudioVolume *volume,
   GList          *sources = NULL;
   GList          *list = NULL;
   guint           i = 0;
+  gchar          *name;
+  gboolean        available;
 
 #ifdef HAVE_MPRIS2
+  guint           num_players;
   gchar         **players;
   gchar          *title = NULL;
   gchar          *artist = NULL;
@@ -477,6 +488,9 @@ pulseaudio_menu_new (PulseaudioVolume *volume,
   menu->config = config;
   menu->mpris = mpris;
   menu->button = widget;
+  menu->connection_changed_id =
+    g_signal_connect_swapped (G_OBJECT (menu->volume), "connection-changed",
+                              G_CALLBACK (pulseaudio_menu_connection_changed), menu);
   menu->volume_changed_id =
     g_signal_connect_swapped (G_OBJECT (menu->volume), "volume-changed",
                               G_CALLBACK (pulseaudio_menu_volume_changed), menu);
@@ -507,7 +521,8 @@ pulseaudio_menu_new (PulseaudioVolume *volume,
           device_mi = device_menu_item_new_with_label(_("Output"));
           for (list = sources; list != NULL; list = g_list_next(list))
             {
-              device_menu_item_add_device(DEVICE_MENU_ITEM(device_mi), (gchar *)list->data, pulseaudio_volume_get_output_by_name(menu->volume, list->data));
+              name = pulseaudio_volume_get_output_by_name(menu->volume, list->data, &available);
+              device_menu_item_add_device(DEVICE_MENU_ITEM(device_mi), (gchar *)list->data, name, available);
             }
 
           device_menu_item_set_device_by_name(DEVICE_MENU_ITEM(device_mi), pulseaudio_volume_get_default_output(menu->volume));
@@ -546,7 +561,8 @@ pulseaudio_menu_new (PulseaudioVolume *volume,
           device_mi = device_menu_item_new_with_label(_("Input"));
           for (list = sources; list != NULL; list = g_list_next(list))
             {
-              device_menu_item_add_device(DEVICE_MENU_ITEM(device_mi), (gchar *)list->data, pulseaudio_volume_get_input_by_name(menu->volume, list->data));
+              name = pulseaudio_volume_get_input_by_name(menu->volume, list->data, &available);
+              device_menu_item_add_device(DEVICE_MENU_ITEM(device_mi), (gchar *)list->data, name, available);
             }
 
           device_menu_item_set_device_by_name(DEVICE_MENU_ITEM(device_mi), pulseaudio_volume_get_default_input(menu->volume));
@@ -568,98 +584,109 @@ pulseaudio_menu_new (PulseaudioVolume *volume,
 #ifdef HAVE_MPRIS2
   if (pulseaudio_config_get_enable_mpris (menu->config))
     {
-      players = pulseaudio_config_get_mpris_players (menu->config);
+      players = pulseaudio_config_get_known_players (menu->config);
       if (players != NULL)
         {
-          for (i = 0; i < g_strv_length (players); i++)
+          num_players = g_strv_length (players);
+          for (i = 0; i < num_players; i++)
             {
-              if (pulseaudio_config_player_blacklist_lookup (menu->config, players[i]))
+              if (pulseaudio_config_player_ignored_lookup (menu->config, players[i]))
+                continue;
+
+              is_running = pulseaudio_mpris_get_player_snapshot (menu->mpris,
+                                                                 players[i],
+                                                                 &title,
+                                                                 &artist,
+                                                                 &is_playing,
+                                                                 &is_stopped,
+                                                                 &can_play,
+                                                                 &can_pause,
+                                                                 &can_go_previous,
+                                                                 &can_go_next,
+                                                                 &can_raise,
+                                                                 &playlists);
+
+              if (!is_running && !pulseaudio_config_player_persistent_lookup (menu->config, players[i]))
                 continue;
 
               mi = mpris_menu_item_new_from_player_name (players[i]);
-              if (mi != NULL)
+              if (mi == NULL)
                 {
-                  if (pulseaudio_mpris_get_player_snapshot (menu->mpris,
-                                                            players[i],
-                                                            &title,
-                                                            &artist,
-                                                            &is_running,
-                                                            &is_playing,
-                                                            &is_stopped,
-                                                            &can_play,
-                                                            &can_pause,
-                                                            &can_go_previous,
-                                                            &can_go_next,
-                                                            &can_raise,
-                                                            &playlists))
+                  if (is_running)
                     {
-                      mpris_menu_item_set_is_running (MPRIS_MENU_ITEM (mi), is_running);
-                      mpris_menu_item_set_title (MPRIS_MENU_ITEM (mi), title);
-                      mpris_menu_item_set_artist (MPRIS_MENU_ITEM (mi), artist);
-
-                      mpris_menu_item_set_can_raise (MPRIS_MENU_ITEM (mi), can_raise);
-                      mpris_menu_item_set_can_raise_wnck (MPRIS_MENU_ITEM (mi), pulseaudio_config_get_can_raise_wnck (menu->config));
-
-                      mpris_menu_item_set_can_play (MPRIS_MENU_ITEM (mi), can_play);
-                      mpris_menu_item_set_can_pause (MPRIS_MENU_ITEM (mi), can_pause);
-
-                      mpris_menu_item_set_can_go_previous (MPRIS_MENU_ITEM (mi), can_go_previous);
-                      mpris_menu_item_set_can_go_next (MPRIS_MENU_ITEM (mi), can_go_next);
-
-                      mpris_menu_item_set_is_playing (MPRIS_MENU_ITEM (mi), is_playing);
-                      mpris_menu_item_set_is_stopped (MPRIS_MENU_ITEM (mi), is_stopped);
-
-                      if (title != NULL)
-                        g_free (title);
-                      if (artist != NULL)
-                        g_free (artist);
+                      if (playlists)
+                        g_list_free (playlists);
+                      g_free (title);
+                      g_free (artist);
                     }
-                  else
-                    {
-                      mpris_menu_item_set_is_running (MPRIS_MENU_ITEM (mi), FALSE);
-                      mpris_menu_item_set_is_stopped (MPRIS_MENU_ITEM (mi), TRUE);
-                    }
-
-                  g_signal_connect (mi, "media-notify", G_CALLBACK (media_notify_cb), menu);
-                  g_signal_connect (menu->mpris, "update", G_CALLBACK (mpris_update_cb), mi);
-                  g_signal_connect (mi, "destroy", G_CALLBACK(item_destroy_cb), menu);
-
-                  gtk_widget_show(mi);
-                  gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-                  if (playlists != NULL)
-                    {
-                      if (g_list_length(playlists) > 0)
-                      {
-                        mi = gtk_menu_item_new_with_label(_("Choose Playlist"));
-                        gtk_widget_show(mi);
-                        gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-                        submenu = gtk_menu_new();
-                        gtk_menu_item_set_submenu(GTK_MENU_ITEM(mi), submenu);
-
-                        for (list = playlists; list != NULL; list = g_list_next(list))
-                        {
-                          mi = gtk_menu_item_new_with_label((gchar *)list->data);
-                          gtk_widget_show(mi);
-                          gtk_menu_shell_append(GTK_MENU_SHELL(submenu), mi);
-
-                          g_object_set_data (G_OBJECT(mi), "player", g_strdup (players[i]));
-                          g_object_set_data (G_OBJECT(mi), "playlist", g_strdup ((gchar *)list->data));
-
-                          g_signal_connect_swapped(G_OBJECT(mi), "activate", G_CALLBACK (pulseaudio_menu_activate_playlist), menu);
-                        }
-                      }
-
-                      g_list_free(playlists);
-                      playlists = NULL;
-                    }
-
-                  /* separator */
-                  mi = gtk_separator_menu_item_new ();
-                  gtk_widget_show (mi);
-                  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+                  continue;
                 }
+
+              if (is_running)
+                {
+                  mpris_menu_item_set_is_running (MPRIS_MENU_ITEM (mi), TRUE);
+
+                  mpris_menu_item_set_title (MPRIS_MENU_ITEM (mi), title);
+                  mpris_menu_item_set_artist (MPRIS_MENU_ITEM (mi), artist);
+
+                  mpris_menu_item_set_can_raise (MPRIS_MENU_ITEM (mi), can_raise);
+                  mpris_menu_item_set_can_raise_wnck (MPRIS_MENU_ITEM (mi), pulseaudio_config_get_can_raise_wnck (menu->config));
+
+                  mpris_menu_item_set_can_play (MPRIS_MENU_ITEM (mi), can_play);
+                  mpris_menu_item_set_can_pause (MPRIS_MENU_ITEM (mi), can_pause);
+
+                  mpris_menu_item_set_can_go_previous (MPRIS_MENU_ITEM (mi), can_go_previous);
+                  mpris_menu_item_set_can_go_next (MPRIS_MENU_ITEM (mi), can_go_next);
+
+                  mpris_menu_item_set_is_playing (MPRIS_MENU_ITEM (mi), is_playing);
+                  mpris_menu_item_set_is_stopped (MPRIS_MENU_ITEM (mi), is_stopped);
+
+                  g_free (title);
+                  g_free (artist);
+                }
+              else
+                {
+                  mpris_menu_item_set_is_running (MPRIS_MENU_ITEM (mi), FALSE);
+                }
+
+                g_signal_connect (mi, "media-notify", G_CALLBACK (media_notify_cb), menu);
+                g_signal_connect (menu->mpris, "update", G_CALLBACK (mpris_update_cb), mi);
+                g_signal_connect (mi, "destroy", G_CALLBACK(item_destroy_cb), menu);
+
+                gtk_widget_show(mi);
+                gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+
+                if (playlists != NULL)
+                  {
+                    if (g_list_length(playlists) > 0)
+                    {
+                      mi = gtk_menu_item_new_with_label(_("Choose Playlist"));
+                      gtk_widget_show(mi);
+                      gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+
+                      submenu = gtk_menu_new();
+                      gtk_menu_item_set_submenu(GTK_MENU_ITEM(mi), submenu);
+
+                      for (list = playlists; list != NULL; list = g_list_next(list))
+                      {
+                        mi = gtk_menu_item_new_with_label((gchar *)list->data);
+                        gtk_widget_show(mi);
+                        gtk_menu_shell_append(GTK_MENU_SHELL(submenu), mi);
+
+                        g_object_set_data (G_OBJECT(mi), "player", g_strdup (players[i]));
+                        g_object_set_data (G_OBJECT(mi), "playlist", g_strdup ((gchar *)list->data));
+
+                        g_signal_connect_swapped(G_OBJECT(mi), "activate", G_CALLBACK (pulseaudio_menu_activate_playlist), menu);
+                      }
+                    }
+
+                    g_list_free(playlists);
+                  }
+
+                /* separator */
+                mi = gtk_separator_menu_item_new ();
+                gtk_widget_show (mi);
+                gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
             }
 
           g_strfreev (players);
