@@ -63,7 +63,6 @@ struct _PulseaudioMprisPlayer
   gboolean          can_pause;
   gboolean          can_play;
   gboolean          can_raise;
-  gboolean          can_launch;
 
   PlaybackStatus    playback_status;
 
@@ -76,6 +75,8 @@ struct _PulseaudioMprisPlayer
 #elif defined (HAVE_WNCK)
   gulong            xid;
 #endif
+
+  gint64            timestamp;
 };
 
 struct _PulseaudioMprisPlayerClass
@@ -352,7 +353,8 @@ pulseaudio_mpris_player_raise_wnck (PulseaudioMprisPlayer *player)
 
 void
 pulseaudio_mpris_player_call_player_method (PulseaudioMprisPlayer *player,
-                                            const gchar           *method)
+                                            const gchar           *method,
+                                            gboolean               update_timestamp)
 {
   GDBusMessage *message;
   GError       *error = NULL;
@@ -394,6 +396,9 @@ pulseaudio_mpris_player_call_player_method (PulseaudioMprisPlayer *player,
     }
 
   g_object_unref (message);
+
+  if (update_timestamp)
+    player->timestamp = g_get_monotonic_time ();
 }
 
 
@@ -603,7 +608,8 @@ pulseaudio_mpris_player_parse_media_player_properties (PulseaudioMprisPlayer *pl
   GVariantIter  iter;
   GVariant     *value;
   const gchar  *key;
-  const gchar  *filename = NULL;
+  const gchar  *name_for_desktop = NULL;
+  const gchar  *identify = NULL;
 
   g_variant_iter_init (&iter, properties);
 
@@ -615,10 +621,22 @@ pulseaudio_mpris_player_parse_media_player_properties (PulseaudioMprisPlayer *pl
         }
       else if (0 == g_ascii_strcasecmp (key, "DesktopEntry"))
         {
-          filename = g_variant_get_string(value, NULL);
-          pulseaudio_mpris_player_set_details_from_desktop (player, filename);
+          name_for_desktop = g_variant_get_string(value, NULL);
+        }
+      else if (0 == g_ascii_strcasecmp (key, "Identity"))
+        {
+          identify = g_variant_get_string(value, NULL);
         }
     }
+
+  if (name_for_desktop == NULL && identify != NULL && *identify != '\0')
+    player->player_label = g_strdup (identify);
+
+  if (name_for_desktop == NULL)
+    name_for_desktop = player->player_label;
+  if (name_for_desktop == NULL)
+    name_for_desktop = player->player;
+  pulseaudio_mpris_player_set_details_from_desktop (player, name_for_desktop);
 }
 
 
@@ -689,18 +707,7 @@ pulseaudio_mpris_player_on_dbus_connected (GDBusConnection *connection,
   PulseaudioMprisPlayer *player = user_data;
 
   player->connected = TRUE;
-
-  /* Notify that connect to a player.*/
-  g_signal_emit (player, signals[CONNECTION], 0, player->connected);
-
-  /* And informs the current status of the player */
-  reply = pulseaudio_mpris_player_get_all_player_properties (player);
-  if (reply)
-    {
-      pulseaudio_mpris_player_parse_player_properties (player, reply);
-      g_variant_unref (reply);
-    }
-
+  player->timestamp = g_get_monotonic_time ();
 
   /* Media player properties */
   reply = pulseaudio_mpris_player_get_all_media_player_properties (player);
@@ -710,6 +717,19 @@ pulseaudio_mpris_player_on_dbus_connected (GDBusConnection *connection,
       g_variant_unref (reply);
     }
 
+  if (player->player_label == NULL)
+    pulseaudio_mpris_player_set_details_from_desktop (player, player->player);
+
+  /* Notify that connect to a player */
+  g_signal_emit (player, signals[CONNECTION], 0, player->connected);
+
+  /* Inform the current status of the player */
+  reply = pulseaudio_mpris_player_get_all_player_properties (player);
+  if (reply)
+    {
+      pulseaudio_mpris_player_parse_player_properties (player, reply);
+      g_variant_unref (reply);
+    }
 
   /* Playlists */
   reply = pulseaudio_mpris_player_playlists_get_playlists (player);
@@ -733,33 +753,19 @@ pulseaudio_mpris_player_on_dbus_lost (GDBusConnection *connection,
 {
   PulseaudioMprisPlayer *player = user_data;
 
-  /* Interface MediaPlayer2.Player */
-  player->playback_status = STOPPED;
-  player->can_go_next     = FALSE;
-  player->can_go_previous = FALSE;
-  player->can_play        = FALSE;
-  player->can_pause       = FALSE;
-  player->can_raise       = FALSE;
+  if (!player->connected)
+    return;
+
   player->connected       = FALSE;
 
-  if (player->title != NULL)
-    g_free (player->title);
-  if (player->artist != NULL)
-    g_free (player->artist);
-
-  player->title           = NULL;
-  player->artist          = NULL;
-#ifdef HAVE_WNCK
-  player->xid             = 0L;
-#endif
-
+  /* This will delete the object */
   g_signal_emit (player, signals[CONNECTION], 0, player->connected);
 }
 
 
 
-static gchar *
-find_desktop_entry (const gchar *player_name)
+gchar *
+pulseaudio_mpris_player_find_desktop_entry (const gchar *player_name)
 {
   GKeyFile  *key_file;
   gchar     *file = NULL;
@@ -811,17 +817,17 @@ pulseaudio_mpris_player_set_details_from_desktop (PulseaudioMprisPlayer *player,
   gchar     *full_path = NULL;
   gchar     *filename = NULL;
 
-  filename = find_desktop_entry (player_name);
+  filename = pulseaudio_mpris_player_find_desktop_entry (player_name);
 
-  if (player->player_label != NULL)
-    g_free (player->player_label);
-  if (player->icon_name != NULL)
-    g_free (player->icon_name);
+  g_free (player->icon_name);
+  g_free (player->full_path);
 
   if (filename == NULL)
     {
-      player->player_label = g_strdup (player->player);
+      if (player->player_label == NULL)
+        player->player_label = g_strdup (player->player);
       player->icon_name = g_strdup ("applications-multimedia");
+      player->full_path = NULL;
       return;
     }
 
@@ -831,25 +837,18 @@ pulseaudio_mpris_player_set_details_from_desktop (PulseaudioMprisPlayer *player,
   key_file = g_key_file_new();
   if (g_key_file_load_from_data_dirs (key_file, file, &full_path, G_KEY_FILE_NONE, NULL))
     {
-      gchar *name = g_key_file_get_locale_string (key_file, "Desktop Entry", "Name", NULL, NULL);
-      gchar *icon_name = g_key_file_get_string (key_file, "Desktop Entry", "Icon", NULL);
-
-      player->player_label = g_strdup (name);
-      player->icon_name = g_strdup (icon_name);
-
-      g_free (name);
-      g_free (icon_name);
+      if (player->player_label == NULL)
+        player->player_label = g_key_file_get_locale_string (key_file, "Desktop Entry", "Name", NULL, NULL);
+      player->icon_name = g_key_file_get_string (key_file, "Desktop Entry", "Icon", NULL);
     }
   else
     {
-      player->player_label = g_strdup (player->player);
+      if (player->player_label == NULL)
+        player->player_label = g_strdup (player->player);
       player->icon_name = g_strdup ("applications-multimedia");
     }
 
-  if (full_path != NULL) {
-    player->full_path = g_strdup (full_path);
-    g_free (full_path);
-  }
+  player->full_path = full_path;
 
   g_key_file_free (key_file);
   g_free (file);
@@ -857,50 +856,6 @@ pulseaudio_mpris_player_set_details_from_desktop (PulseaudioMprisPlayer *player,
 
 
 
-static void
-pulseaudio_mpris_player_set_player (PulseaudioMprisPlayer *player,
-                                    const gchar           *player_name)
-{
-  /* Disconnect dbus */
-  if (player->watch_id)
-    {
-      g_bus_unwatch_name (player->watch_id);
-      player->watch_id = 0;
-    }
-  if (player->dbus_props_proxy != NULL)
-    {
-      g_object_unref (player->dbus_props_proxy);
-      player->dbus_props_proxy = NULL;
-    }
-  if (player->dbus_player_proxy != NULL)
-    {
-      g_object_unref (player->dbus_player_proxy);
-      player->dbus_player_proxy = NULL;
-    }
-  if (player->dbus_playlists_proxy != NULL)
-    {
-      g_object_unref (player->dbus_playlists_proxy);
-      player->dbus_playlists_proxy = NULL;
-    }
-
-  /* Clean player */
-  if (player->player != NULL)
-    {
-      g_free (player->player);
-      player->player = NULL;
-    }
-
-  /* Set new player and connect again */
-  player->player = g_strdup(player_name);
-
-  pulseaudio_mpris_player_set_details_from_desktop (player, player_name);
-  pulseaudio_mpris_player_dbus_connect (player);
-
-  player->can_launch = player->full_path != NULL;
-}
-
-
-
 static void
 pulseaudio_mpris_player_dbus_connect (PulseaudioMprisPlayer *player)
 {
@@ -991,6 +946,14 @@ pulseaudio_mpris_player_dbus_connect (PulseaudioMprisPlayer *player)
 
 
 
+gint64
+pulseaudio_mpris_player_get_timestamp (PulseaudioMprisPlayer *player)
+{
+  return player->timestamp;
+}
+
+
+
 const gchar *
 pulseaudio_mpris_player_get_player (PulseaudioMprisPlayer *player)
 {
@@ -1103,47 +1066,11 @@ pulseaudio_mpris_player_can_raise (PulseaudioMprisPlayer *player)
 
 
 
-gboolean
-pulseaudio_mpris_player_can_launch (PulseaudioMprisPlayer *player)
-{
-  return player->can_launch;
-}
-
-
-
-gboolean
-pulseaudio_mpris_player_is_equal (PulseaudioMprisPlayer *a,
-                                  PulseaudioMprisPlayer *b)
-{
-  return g_strcmp0 (pulseaudio_mpris_player_get_player_title (a), pulseaudio_mpris_player_get_player_title (b)) == 0;
-}
-
-
-
 static void
 pulseaudio_mpris_player_init (PulseaudioMprisPlayer *player)
 {
-  player->dbus_connection   = NULL;
-  player->dbus_name         = NULL;
-  player->dbus_props_proxy  = NULL;
-  player->dbus_player_proxy = NULL;
-  player->dbus_playlists_proxy = NULL;
-  player->connected         = FALSE;
-
-  player->title             = NULL;
-  player->artist            = NULL;
-  player->full_path         = NULL;
-
-  player->can_go_next       = FALSE;
-  player->can_go_previous   = FALSE;
-  player->can_pause         = FALSE;
-  player->can_play          = FALSE;
-  player->can_raise         = FALSE;
-
   player->playback_status   = STOPPED;
 
-  player->watch_id          = 0;
-
 #ifdef HAVE_LIBXFCE4WINDOWING
   player->screen = xfw_screen_get_default ();
 #endif
@@ -1158,26 +1085,23 @@ pulseaudio_mpris_player_finalize (GObject *object)
 
   player = PULSEAUDIO_MPRIS_PLAYER (object);
 
-  player->dbus_connection   = NULL;
-  player->dbus_name         = NULL;
-  player->dbus_props_proxy  = NULL;
-  player->dbus_player_proxy = NULL;
-  player->dbus_playlists_proxy = NULL;
-  player->connected         = FALSE;
+  /* Disconnect dbus */
+  if (player->watch_id != 0)
+    g_bus_unwatch_name (player->watch_id);
+  if (player->dbus_props_proxy != NULL)
+    g_object_unref (player->dbus_props_proxy);
+  if (player->dbus_player_proxy != NULL)
+    g_object_unref (player->dbus_player_proxy);
+  if (player->dbus_playlists_proxy != NULL)
+    g_object_unref (player->dbus_playlists_proxy);
 
-  player->title             = NULL;
-  player->artist            = NULL;
-  player->full_path         = NULL;
+  g_free (player->player);
+  g_free (player->player_label);
+  g_free (player->icon_name);
 
-  player->can_go_next       = FALSE;
-  player->can_go_previous   = FALSE;
-  player->can_pause         = FALSE;
-  player->can_play          = FALSE;
-  player->can_raise         = FALSE;
-
-  player->playback_status   = STOPPED;
-
-  player->watch_id          = 0;
+  g_free (player->title);
+  g_free (player->artist);
+  g_free (player->full_path);
 
   if (player->playlists != NULL)
     g_hash_table_destroy (player->playlists);
@@ -1203,17 +1127,16 @@ pulseaudio_mpris_player_new (gchar *name)
     {
       g_message ("Failed to get session bus: %s", gerror->message);
       g_error_free (gerror);
-      gerror = NULL;
+      return NULL;
     }
 
   player = g_object_new (TYPE_PULSEAUDIO_MPRIS_PLAYER, NULL);
 
   player->dbus_connection = gconnection;
+  player->player = g_strdup (name);
+  player->playlists = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
   pulseaudio_mpris_player_dbus_connect (player);
-  pulseaudio_mpris_player_set_player (player, name);
-
-  player->playlists = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
 
   return player;
 }
