@@ -1,4 +1,4 @@
-/*  Copyright (c) 2017 Sean Davis <bluesabre@xfce.org>
+/*  Copyright (c) 2017-2020 Sean Davis <bluesabre@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -52,11 +52,15 @@ typedef struct          _PulseaudioMprisPlayerClass                PulseaudioMpr
 
 PulseaudioMprisPlayer  *pulseaudio_mpris_player_new                (gchar *name);
 
+gchar                  *pulseaudio_mpris_player_find_desktop_entry (const gchar           *player_name);
+
+gint64                  pulseaudio_mpris_player_get_timestamp      (PulseaudioMprisPlayer *player);
 const gchar            *pulseaudio_mpris_player_get_player         (PulseaudioMprisPlayer *player);
 const gchar            *pulseaudio_mpris_player_get_player_title   (PulseaudioMprisPlayer *player);
 const gchar            *pulseaudio_mpris_player_get_icon_name      (PulseaudioMprisPlayer *player);
 const gchar            *pulseaudio_mpris_player_get_title          (PulseaudioMprisPlayer *player);
 const gchar            *pulseaudio_mpris_player_get_artist         (PulseaudioMprisPlayer *player);
+const gchar            *pulseaudio_mpris_player_get_full_path      (PulseaudioMprisPlayer *player);
 
 gboolean                pulseaudio_mpris_player_is_connected       (PulseaudioMprisPlayer *player);
 gboolean                pulseaudio_mpris_player_is_playing         (PulseaudioMprisPlayer *player);
@@ -68,11 +72,9 @@ gboolean                pulseaudio_mpris_player_can_go_previous    (PulseaudioMp
 gboolean                pulseaudio_mpris_player_can_go_next        (PulseaudioMprisPlayer *player);
 gboolean                pulseaudio_mpris_player_can_raise          (PulseaudioMprisPlayer *player);
 
-gboolean                pulseaudio_mpris_player_is_equal           (PulseaudioMprisPlayer *a,
-                                                                    PulseaudioMprisPlayer *b);
-
 void                    pulseaudio_mpris_player_call_player_method (PulseaudioMprisPlayer *player,
-                                                                    const gchar           *method);
+                                                                    const gchar           *method,
+                                                                    gboolean               update_timestamp);
 
 GList                  *pulseaudio_mpris_player_get_playlists      (PulseaudioMprisPlayer *player);
 void                    pulseaudio_mpris_player_activate_playlist  (PulseaudioMprisPlayer *player,
