@@ -87,6 +87,7 @@ struct _PulseaudioVolume
   /* Device management */
   GHashTable           *sinks;
   GHashTable           *sources;
+  GHashTable           *source_idx_to_name;
 
   guint                 sink_index;
   guint                 source_index;
@@ -102,8 +103,17 @@ struct _PulseaudioVolumeClass
 
 
 
+typedef struct
+{
+  gchar                *description;
+  gboolean              available;
+} PulseAudioDeviceInfo;
+
+
+
 enum
 {
+  CONNECTION_CHANGED,
   VOLUME_CHANGED,
   VOLUME_MIC_CHANGED,
   RECORDING_CHANGED,
@@ -118,6 +128,19 @@ G_DEFINE_TYPE (PulseaudioVolume, pulseaudio_volume, G_TYPE_OBJECT)
 
 
 
+static void
+pulseaudio_volume_free_device_info (gpointer data)
+{
+  PulseAudioDeviceInfo *device_info = (PulseAudioDeviceInfo *) data;
+  if (device_info)
+    {
+      g_free (device_info->description);
+      g_free (device_info);
+    }
+}
+
+
+
 static void
 pulseaudio_volume_class_init (PulseaudioVolumeClass *klass)
 {
@@ -126,6 +149,14 @@ pulseaudio_volume_class_init (PulseaudioVolumeClass *klass)
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = pulseaudio_volume_finalize;
 
+  pulseaudio_volume_signals[CONNECTION_CHANGED] =
+    g_signal_new (g_intern_static_string ("connection-changed"),
+                  G_TYPE_FROM_CLASS (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
   pulseaudio_volume_signals[VOLUME_CHANGED] =
     g_signal_new (g_intern_static_string ("volume-changed"),
                   G_TYPE_FROM_CLASS (gobject_class),
@@ -170,8 +201,9 @@ pulseaudio_volume_init (PulseaudioVolume *volume)
 
   volume->pa_mainloop = pa_glib_mainloop_new (NULL);
 
-  volume->sinks = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
-  volume->sources = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
+  volume->sinks = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, pulseaudio_volume_free_device_info);
+  volume->sources = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, pulseaudio_volume_free_device_info);
+  volume->source_idx_to_name = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
 
   pulseaudio_volume_connect (volume);
 }
@@ -194,6 +226,7 @@ pulseaudio_volume_finalize (GObject *object)
 
   g_hash_table_destroy (volume->sinks);
   g_hash_table_destroy (volume->sources);
+  g_hash_table_destroy (volume->source_idx_to_name);
 
   pa_glib_mainloop_free (volume->pa_mainloop);
 
@@ -214,6 +247,16 @@ pulseaudio_volume_sink_info_cb (pa_context         *context,
 
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
 
+  if (eol > 0)
+    {
+      if (!volume->sink_connected)
+        {
+          g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [CONNECTION_CHANGED], 0);
+          volume->sink_connected = TRUE;
+        }
+      return;
+    }
+
   if (i == NULL)
     return;
 
@@ -245,7 +288,6 @@ pulseaudio_volume_sink_info_cb (pa_context         *context,
     }
 
   pulseaudio_debug ("volume: %f, muted: %d", vol, muted);
-  volume->sink_connected = TRUE;
 }
 
 
@@ -261,6 +303,16 @@ pulseaudio_volume_source_info_cb (pa_context           *context,
 
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
 
+  if (eol > 0)
+    {
+      if (!volume->source_connected)
+        {
+          g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [CONNECTION_CHANGED], 0);
+          volume->source_connected = TRUE;
+        }
+      return;
+    }
+
   if (i == NULL)
     return;
 
@@ -293,7 +345,6 @@ pulseaudio_volume_source_info_cb (pa_context           *context,
     }
 
   pulseaudio_debug ("volume mic: %f, muted mic: %d", vol_mic, muted_mic);
-  volume->source_connected = TRUE;
 }
 
 
@@ -328,6 +379,7 @@ pulseaudio_volume_sink_source_check (PulseaudioVolume *volume,
 
   g_hash_table_remove_all (volume->sinks);
   g_hash_table_remove_all (volume->sources);
+  g_hash_table_remove_all (volume->source_idx_to_name);
 
   pa_context_get_sink_info_list (volume->pa_context, pulseaudio_volume_get_sink_list_cb, volume);
   pa_context_get_source_info_list (volume->pa_context, pulseaudio_volume_get_source_list_cb, volume);
@@ -335,6 +387,16 @@ pulseaudio_volume_sink_source_check (PulseaudioVolume *volume,
 
 
 
+static void
+pulseaudio_volume_update_recording_state (PulseaudioVolume             *volume,
+                                          pa_context                   *context)
+{
+  volume->recording = FALSE;
+  pa_context_get_source_output_info_list (context, pulseaudio_volume_get_source_output_info_cb, volume);
+}
+
+
+
 static void
 pulseaudio_volume_subscribe_cb (pa_context                   *context,
                                 pa_subscription_event_type_t  t,
@@ -361,10 +423,8 @@ pulseaudio_volume_subscribe_cb (pa_context                   *context,
       break;
 
     case PA_SUBSCRIPTION_EVENT_SOURCE_OUTPUT :
-      volume->recording = FALSE;
-      pa_context_get_source_output_info_list (context, pulseaudio_volume_get_source_output_info_cb, volume);
-
       pulseaudio_volume_sink_source_check (volume, context);
+      pulseaudio_volume_update_recording_state (volume, context);
       pulseaudio_debug ("received source output event");
       break;
 
@@ -388,6 +448,7 @@ pulseaudio_volume_get_sink_list_cb (pa_context         *context,
                                     void               *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  PulseAudioDeviceInfo *device_info;
 
   if (i == NULL)
     return;
@@ -395,7 +456,10 @@ pulseaudio_volume_get_sink_list_cb (pa_context         *context,
   if (eol > 0)
     return;
 
-  g_hash_table_insert (volume->sinks, g_strdup (i->name), g_strdup (i->description));
+  device_info = g_new (PulseAudioDeviceInfo, 1);
+  device_info->description = g_strdup (i->description);
+  device_info->available = (!i->active_port || i->active_port->available != PA_PORT_AVAILABLE_NO);
+  g_hash_table_insert (volume->sinks, g_strdup (i->name), device_info);
 }
 
 
@@ -407,6 +471,7 @@ pulseaudio_volume_get_source_output_info_cb (pa_context                  *contex
                                              void                        *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  const gchar      *str;
 
   if (eol > 0)
     {
@@ -420,19 +485,26 @@ pulseaudio_volume_get_source_output_info_cb (pa_context                  *contex
 
   if (i)
     {
-      if (g_strcmp0("org.PulseAudio.pavucontrol",
-                    pa_proplist_gets (i->proplist, PA_PROP_APPLICATION_ID)) == 0)
-        {
-          pulseaudio_debug ("Don't show recording indicator for pavucontrol");
+      /* Don't show recording indicator when it's not connected to the source */
+      if (i->source == PA_INVALID_INDEX)
           return;
-        }
+
+      /* Don't show recording indicator if the source output doesn't belong to any client */
+      if (i->client == PA_INVALID_INDEX)
+        return;
+
+      /* Don't show recording indicator for pavucontrol */
+      str = pa_proplist_gets (i->proplist, PA_PROP_APPLICATION_ID);
+      if (str && g_strcmp0("org.PulseAudio.pavucontrol", str) == 0)
+        return;
+
+      /* Don't show recording indicator for non-default monitors */
+      str = g_hash_table_lookup (volume->source_idx_to_name, GINT_TO_POINTER (i->source));
+      if (str && g_str_has_suffix (str, ".monitor") && g_strcmp0 (str, volume->default_source_name) != 0)
+        return;
 
       volume->recording = TRUE;
     }
-  else
-    {
-      volume->recording = FALSE;
-    }
 }
 
 
@@ -443,6 +515,7 @@ pulseaudio_volume_get_source_list_cb (pa_context           *context,
                                       void                 *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
+  PulseAudioDeviceInfo *device_info;
 
   if (i == NULL)
     return;
@@ -450,30 +523,16 @@ pulseaudio_volume_get_source_list_cb (pa_context           *context,
   if (eol > 0)
     return;
 
-  /* Ignore sink monitors if it is not a default sink, not relevant for users */
+  g_hash_table_insert (volume->source_idx_to_name, GINT_TO_POINTER (i->index), g_strdup (i->name));
+
+  /* Ignore sink monitors if it is not a default source, not relevant for users */
   if (i->monitor_of_sink != PA_INVALID_INDEX && g_strcmp0 (i->name, volume->default_source_name) != 0)
     return;
 
-  g_hash_table_insert (volume->sources, g_strdup (i->name), g_strdup (i->description));
-}
-
-
-
-static void
-pulseaudio_volume_get_server_info_cb (pa_context           *context,
-                                      const pa_server_info *i,
-                                      void                 *userdata)
-{
-  PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-
-  if (i == NULL)
-    return;
-
-  g_free (volume->default_sink_name);
-  g_free (volume->default_source_name);
-
-  volume->default_sink_name = g_strdup (i->default_sink_name);
-  volume->default_source_name = g_strdup (i->default_source_name);
+  device_info = g_new (PulseAudioDeviceInfo, 1);
+  device_info->description = g_strdup (i->description);
+  device_info->available = (!i->active_port || i->active_port->available != PA_PORT_AVAILABLE_NO);
+  g_hash_table_insert (volume->sources, g_strdup (i->name), device_info);
 }
 
 
@@ -492,21 +551,16 @@ pulseaudio_volume_context_state_cb (pa_context *context,
 
       pulseaudio_debug ("PulseAudio connection established");
       volume->connected = TRUE;
-      // Check current sink and source volume manually. PA sink events usually not emitted.
-      pulseaudio_volume_sink_source_check (volume, context);
-
-      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
-      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
-      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [RECORDING_CHANGED], 0, FALSE);
-
       volume->sink_connected = FALSE;
       volume->source_connected = FALSE;
 
-      pa_context_get_server_info (volume->pa_context, pulseaudio_volume_get_server_info_cb, volume);
-
-      // Check here if recording is active
-      pa_context_get_source_output_info_list (context, pulseaudio_volume_get_source_output_info_cb, volume);
+      // Check current sink and source volume manually. PA sink events usually not emitted.
+      pulseaudio_volume_sink_source_check (volume, context);
 
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [CONNECTION_CHANGED], 0);
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [RECORDING_CHANGED], 0, FALSE);
       break;
 
     case PA_CONTEXT_FAILED       :
@@ -514,6 +568,8 @@ pulseaudio_volume_context_state_cb (pa_context *context,
       g_warning ("Disconnected from the PulseAudio server. Attempting to reconnect in 5 seconds...");
       volume->pa_context = NULL;
       volume->connected = FALSE;
+      volume->sink_connected = FALSE;
+      volume->source_connected = FALSE;
       volume->volume = 0.0;
       volume->muted = FALSE;
       volume->recording = FALSE;
@@ -522,12 +578,14 @@ pulseaudio_volume_context_state_cb (pa_context *context,
       volume->base_volume_mic = 0.0;
       volume->muted_mic = FALSE;
 
+      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [CONNECTION_CHANGED], 0);
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [RECORDING_CHANGED], 0, FALSE);
 
       g_hash_table_remove_all (volume->sinks);
       g_hash_table_remove_all (volume->sources);
+      g_hash_table_remove_all (volume->source_idx_to_name);
 
       if (volume->reconnect_timer_id == 0)
         volume->reconnect_timer_id = g_timeout_add_seconds
@@ -608,26 +666,6 @@ pulseaudio_volume_get_connected (PulseaudioVolume *volume)
 
 
 
-gboolean
-pulseaudio_volume_get_sink_connected (PulseaudioVolume *volume)
-{
-  g_return_val_if_fail (IS_PULSEAUDIO_VOLUME (volume), FALSE);
-
-  return volume->sink_connected;
-}
-
-
-
-gboolean
-pulseaudio_volume_get_source_connected (PulseaudioVolume *volume)
-{
-  g_return_val_if_fail (IS_PULSEAUDIO_VOLUME (volume), FALSE);
-
-  return volume->source_connected;
-}
-
-
-
 static gdouble
 pulseaudio_volume_v2d (PulseaudioVolume *volume,
                        pa_volume_t       pa_volume)
@@ -930,14 +968,37 @@ pulseaudio_volume_set_volume_mic (PulseaudioVolume *volume,
 
 
 
+static gchar *
+pulseaudio_volume_get_description (GHashTable  *hash_table,
+                                   const gchar *key,
+                                   gboolean    *available)
+{
+  PulseAudioDeviceInfo *device_info = g_hash_table_lookup (hash_table, key);
+
+  if (device_info)
+    {
+      if (available)
+        *available = device_info->available;
+      return device_info->description;
+    }
+  else if (available)
+    {
+      *available = FALSE;
+    }
+
+  return NULL;
+}
+
+
+
 static gint
 sort_device_list (gchar *a,
                   gchar *b,
                   void  *hash_table)
 {
   GHashTable *table = (GHashTable *)hash_table;
-  gchar      *a_val = (gchar *) g_hash_table_lookup (table, a);
-  gchar      *b_val = (gchar *) g_hash_table_lookup (table, b);
+  gchar      *a_val = pulseaudio_volume_get_description (table, a, NULL);
+  gchar      *b_val = pulseaudio_volume_get_description (table, b, NULL);
   return g_strcmp0 (a_val, b_val);
 }
 
@@ -961,10 +1022,11 @@ pulseaudio_volume_get_output_list (PulseaudioVolume *volume)
 
 gchar *
 pulseaudio_volume_get_output_by_name (PulseaudioVolume *volume,
-                                      gchar            *name)
+                                      const gchar      *name,
+                                      gboolean         *available)
 {
   g_return_val_if_fail (IS_PULSEAUDIO_VOLUME (volume), NULL);
-  return (gchar *) g_hash_table_lookup (volume->sinks, name);
+  return pulseaudio_volume_get_description (volume->sinks, name, available);
 }
 
 
@@ -987,10 +1049,11 @@ pulseaudio_volume_get_input_list (PulseaudioVolume *volume)
 
 gchar *
 pulseaudio_volume_get_input_by_name (PulseaudioVolume *volume,
-                                     gchar            *name)
+                                     const gchar      *name,
+                                     gboolean         *available)
 {
   g_return_val_if_fail (IS_PULSEAUDIO_VOLUME (volume), NULL);
-  return (gchar *) g_hash_table_lookup (volume->sources, name);
+  return pulseaudio_volume_get_description (volume->sources, name, available);
 }
 
 
@@ -1050,6 +1113,9 @@ pulseaudio_volume_set_default_input (PulseaudioVolume *volume,
     {
       g_free (volume->default_source_name);
       volume->default_source_name = g_strdup (name);
+
+      /* Update recording state here, because it uses the "default_source_name" and it's also called on context ready */
+      pulseaudio_volume_update_recording_state (volume, volume->pa_context);
     }
 }
 
